/**
 **	File ......... EventFactory.cpp
 **	Published ....  2005-02-07
 **	Author ....... grymse@alhem.net
**/
/*
Copyright (C) 2004  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <stdincl.h>

#include "client.h"
#include "InitSDL.h"
#include "WinSockInit.h"
#include "Main.h"
#include "commands/ClassFactory.h"
#include "data/ClassFactory.h"
#include "events/ClassFactory.h"
#include "forms/ClassFactory.h"
#include "EventFactory.h"


#undef DEB
#ifdef _DEBUG
#define DEB(x) x
#else
#define DEB(x)
#endif


EventFactory::EventFactory()
{
}


EventFactory::~EventFactory()
{
}


void EventFactory::StateMachine(IOThread *pclIOT)
{
	struct timeval tv;
	char c;

	for(;;)
	{
		switch (m_state)
		{
		case 0:
//DEB(printf("EventFactory state 0\n");)
			if (!pclIOT -> get_iq())
				return;
			pclIOT -> read_input( &c, 1 );
			if (c == 0x02) // stx
				m_state = 1;
			break;
		case 1:
DEB(printf("EventFactory state 1\n");)
			if (pclIOT -> get_iq() < m_l0)
				return;
			pclIOT -> read_input( (char *)&m_id, sizeof(IO_IDTYPE) );
			pclIOT -> read_input( (char *)&m_length, sizeof(IO_LENGTHTYPE) );
			m_state = 2;
			break;
		case 2:
DEB(printf("EventFactory state 2\n");)
			if (pclIOT -> get_iq() < m_length - m_l0)
				return;
			m_state = 0;
			if ( (m_pclMessage = GetClass(m_id)) == NULL)
			{
				fprintf(stderr,"EventFactory, GetClass() failed, id = %d\n",m_id);
				return;
			}
#ifdef TRY_VAR
			m_pclMessage -> ReadVars(pclIOT);
#else
			m_pclMessage -> read_message(pclIOT);
#endif

			if (m_pclMessage -> GetIOFamily() == IOF_FORM)
			{
DEB(printf("Form received - add to input thread form list\n");)
//				it -> AddForm(m_pclMessage);
				SDL_Event e;

				m_pclMessage -> FormInit(pclIOT);
				e.user.type = SDL_USEREVENT;
				e.user.code = SDL_USEREVENT_ADDFORM;
				e.user.data1 = m_pclMessage;
				e.user.data2 = NULL;
				SDL_PushEvent(&e);
			}
			else
			{
				// wait for execute time
				if (m_pclMessage -> GetID() != EVENT_SYNC)
				{
					tv.tv_usec = (m_pclMessage -> GetTicks() - ticks.get_sync()) * 1000;
					long l = tv.tv_usec / 1000;
					tv.tv_sec = tv.tv_usec / 1000000;
					tv.tv_usec %= 1000000;
					if (tv.tv_sec > 0 || tv.tv_usec > 0)
					{
						DEB(printf("timing event (%ld,%ld) %ld ms\n", tv.tv_sec,tv.tv_usec,l);)
						SDL_Delay(l);
					}
				}
				m_pclMessage -> Execute(pclIOT);
			}
			if (m_pclMessage -> DeleteOnExecute())
			{
				delete m_pclMessage;
			}
			break;
		}
	}
}


IOMessage *EventFactory::GetClass(IO_IDTYPE id)
{
	{
		Data::ClassFactory fac;
		IOMessage *pclTemp = fac.GetClass(id);
		if (pclTemp)
		{
			return pclTemp;
		}
	}
	{
		Event::ClassFactory fac;
		IOMessage *pclTemp = fac.GetClass(id);
		if (pclTemp)
		{
			return pclTemp;
		}
	}
	{
		Form::ClassFactory fac;
		IOMessage *pclTemp = fac.GetClass(id);
		if (pclTemp)
		{
			return pclTemp;
		}
	}
	fprintf(stderr,"EventFactory: Unknown event id: %d\n",id);
	exit(-1);
}
